﻿//
// Copyright (C) 2009 Chris Dziemborowicz
//
// This file is part of Orzeszek Timer.
//
// Orzeszek Timer is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// Orzeszek Timer is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Media;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace OrzeszekTimer
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		private Thread updaterThread;
		private WindowInteropHelper interopHelper;

		private bool notified = true;
		private DateTime start;
		private TimeSpan time;

		public MainWindow()
		{
			InitializeComponent();

			if (Settings.Default.WindowSettingsSaved)
				try
				{
					Height = Settings.Default.WindowHeight;
					Width = Settings.Default.WindowWidth;
					Left = Settings.Default.WindowLeft;
					Top = Settings.Default.WindowTop;
					WindowState = Settings.Default.WindowState;
				}
				catch (Exception)
				{
				}

			SetAlarmVolume(Settings.Default.AlarmVolume);

			interopHelper = new WindowInteropHelper(this);
		}

		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			string args = (string)Application.Current.Properties["Args"];
			if (args != null)
				try
				{
					MainTextBox.Text = args;
					MainTextBox.Focus();

					time = FromString(MainTextBox.Text);
					start = DateTime.Now;
					notified = false;

					MainTextBox.Template = (ControlTemplate)Resources["ValidTextBoxTemplate"];
					MainButton.Focus();

					try
					{
						Settings.Default.LastTimeSpan = time;
						Settings.Default.Save();
					}
					catch (Exception)
					{
					}
				}
				catch (Exception)
				{
					MainTextBox.Template = (ControlTemplate)Resources["InvalidTextBoxTemplate"];
				}
			else
			{
				MainTextBox.Text = ToString(Settings.Default.LastTimeSpan.Ticks < 0 ? TimeSpan.Zero : Settings.Default.LastTimeSpan);
				MainTextBox.Focus();
			}
		}

		private void MainTextBox_KeyUp(object sender, KeyEventArgs e)
		{
			if (e.Key == Key.Enter)
				try
				{
					time = FromString(MainTextBox.Text);
					start = DateTime.Now;
					notified = false;

					MainTextBox.Template = (ControlTemplate)Resources["ValidTextBoxTemplate"];
					MainButton.Focus();

					try
					{
						Settings.Default.LastTimeSpan = time;
						Settings.Default.Save();
					}
					catch (Exception)
					{
					}
				}
				catch (Exception)
				{
					MainTextBox.Template = (ControlTemplate)Resources["InvalidTextBoxTemplate"];
				}
			else if (e.Key == Key.Escape)
			{
				MainTextBox.Template = (ControlTemplate)Resources["ValidTextBoxTemplate"];
				MainButton.Focus();
			}
		}

		private void MainTextBox_GotFocus(object sender, RoutedEventArgs e)
		{
			MainTextBox.SelectAll();
		}

		private void MainTextBox_LostFocus(object sender, RoutedEventArgs e)
		{
			if (updaterThread == null)
			{
				updaterThread = new Thread(new ThreadStart(UpdateInterface));
				updaterThread.Start();
			}
		}

		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			try
			{
				Settings.Default.WindowHeight = RestoreBounds.Height;
				Settings.Default.WindowWidth = RestoreBounds.Width;
				Settings.Default.WindowLeft = RestoreBounds.Left;
				Settings.Default.WindowTop = RestoreBounds.Top;
				Settings.Default.WindowState = WindowState;
				Settings.Default.WindowSettingsSaved = true;
				Settings.Default.Save();
			}
			catch (Exception)
			{
			}
		}

		private void Window_Closed(object sender, EventArgs e)
		{
			if (updaterThread != null)
				updaterThread.Abort();
		}

		private void Data_Updated(object sender, EventArgs e)
		{
			if (updaterThread == null)
			{
				updaterThread = new Thread(new ThreadStart(UpdateInterface));
				updaterThread.Start();
			}
		}

		private void UpdateInterface()
		{
			while (true)
			{
				DispatcherOperation op = Dispatcher.BeginInvoke(new Action(delegate()
					{
						DateTime now = DateTime.Now;
						TimeSpan elapsed = now - start;
						TimeSpan remaining = start.AddTicks(time.Ticks) - now;

						MainProgressBar.Value = Math.Min(100.0, 100.0 * elapsed.Ticks / time.Ticks);
						TaskbarUtility.SetProgressValue(interopHelper.Handle, (ulong)MainProgressBar.Value, 100);

						if (!MainTextBox.IsFocused)
							if (remaining.Ticks <= 0)
							{
								MainTextBox.Text = "Timer expired";
								Title = "Orzeszek Timer";

								TaskbarUtility.SetProgressState(interopHelper.Handle, TaskbarProgressState.NoProgress);
							}
							else
							{
								MainTextBox.Text = ToString(remaining);
								Title = MainTextBox.Text;
							}

						if (remaining.Ticks <= 0 && !notified)
							try
							{
								notified = true;
								SoundPlayer sp;

								switch (Settings.Default.AlarmVolume)
								{
									case 0:
										sp = null;
										break;

									case 1:
										sp = new SoundPlayer("Timer Expired (Low).wav");
										break;

									case 2:
										sp = new SoundPlayer("Timer Expired (Medium).wav");
										break;

									case 3:
										sp = new SoundPlayer("Timer Expired (High).wav");
										break;

									default:
										sp = new SoundPlayer("Timer Expired (Medium).wav");
										break;
								}

								if (sp != null)
									sp.Play();
							}
							catch (Exception)
							{
							}
					}
				));

				op.Wait();

				Thread.Sleep(10);
			}
		}

		private static TimeSpan FromString(string s)
		{
			if (string.IsNullOrEmpty(s))
				return TimeSpan.Zero;

			double minutes;
			if (double.TryParse(s, out minutes))
				return TimeSpan.FromMinutes(minutes);

			try
			{
				TimeSpan ts = TimeSpan.Zero;
				string[] parts = Regex.Split(s, @"(?<=\d+\s*[a-zA-Z]+)\s*(?=\d+\s*[a-zA-Z]+)");
				foreach (string p in parts)
					if (!string.IsNullOrEmpty(p))
					{
						string[] subparts = Regex.Split(p, @"(?<=\d+)\s*(?=[a-zA-Z]+)");
						if (subparts.Length == 2)
							switch (subparts[1].ToLower()[0])
							{
								case 'd':
									ts = ts.Add(TimeSpan.FromDays(double.Parse(subparts[0])));
									break;
								case 'h':
									ts = ts.Add(TimeSpan.FromHours(double.Parse(subparts[0])));
									break;
								case 'm':
									ts = ts.Add(TimeSpan.FromMinutes(double.Parse(subparts[0])));
									break;
								case 's':
									ts = ts.Add(TimeSpan.FromSeconds(double.Parse(subparts[0])));
									break;
								default:
									throw new FormatException();
							}
						else
							throw new FormatException();
					}

				return ts;
			}
			catch (Exception)
			{
			}

			try
			{
				TimeSpan ts = TimeSpan.Zero;
				string[] parts = s.Split(new char[] { '.', ':', '-', ' ', '/', '\\', '\f', '\n', '\r', '\t', '\v', '\x85' }, StringSplitOptions.RemoveEmptyEntries);
				for (int i = 0; i < parts.Length; i++)
				{
					double d = double.Parse(parts[i]);
					switch (parts.Length - i)
					{
						case 1:
							ts = ts.Add(TimeSpan.FromSeconds(d));
							break;
						case 2:
							ts = ts.Add(TimeSpan.FromMinutes(d));
							break;
						case 3:
							ts = ts.Add(TimeSpan.FromHours(d));
							break;
						case 4:
							ts = ts.Add(TimeSpan.FromDays(d));
							break;
						default:
							throw new FormatException();
					}
				}

				return ts;
			}
			catch (Exception)
			{
			}

			TimeSpan timeSpan = TimeSpan.Zero;
			if (TimeSpan.TryParse(s, out timeSpan))
				return timeSpan;

			throw new FormatException();
		}

		private static string ToString(TimeSpan ts)
		{
			StringBuilder sb = new StringBuilder();

			if ((int)ts.TotalDays == 1)
				sb.AppendFormat("{0:D}\u00A0day ", (int)ts.TotalDays);
			else if ((int)ts.TotalDays != 0)
				sb.AppendFormat("{0:D}\u00A0days ", (int)ts.TotalDays);

			if ((int)ts.TotalHours == 1)
				sb.AppendFormat("{0:D}\u00A0hour ", ts.Hours);
			else if ((int)ts.TotalHours != 0)
				sb.AppendFormat("{0:D}\u00A0hours ", ts.Hours);

			if ((int)ts.TotalMinutes == 1)
				sb.AppendFormat("{0:D}\u00A0minute ", ts.Minutes);
			else if ((int)ts.TotalMinutes != 0)
				sb.AppendFormat("{0:D}\u00A0minutes ", ts.Minutes);

			if (ts.Seconds == 1)
				sb.AppendFormat("{0:D}\u00A0second", ts.Seconds);
			else
				sb.AppendFormat("{0:D}\u00A0seconds", ts.Seconds);

			return sb.ToString();
		}

		private void NotificationOffMenuItem_Click(object sender, RoutedEventArgs e)
		{
			SetAlarmVolume(0);

			Settings.Default.AlarmVolume = 0;
			Settings.Default.Save();
		}

		private void LowAlarmMenuItem_Click(object sender, RoutedEventArgs e)
		{
			SetAlarmVolume(1);

			Settings.Default.AlarmVolume = 1;
			Settings.Default.Save();
		}

		private void MediumAlarmMenuItem_Click(object sender, RoutedEventArgs e)
		{
			SetAlarmVolume(2);

			Settings.Default.AlarmVolume = 2;
			Settings.Default.Save();
		}

		private void HighAlarmMenuItem_Click(object sender, RoutedEventArgs e)
		{
			SetAlarmVolume(3);

			Settings.Default.AlarmVolume = 3;
			Settings.Default.Save();
		}

		private void SetAlarmVolume(int volume)
		{
			switch (volume)
			{
				case 0:
					NoNotificationMenuItem.IsChecked = true;
					LowAlarmMenuItem.IsChecked = false;
					MediumAlarmMenuItem.IsChecked = false;
					HighAlarmMenuItem.IsChecked = false;
					break;

				case 1:
					NoNotificationMenuItem.IsChecked = false;
					LowAlarmMenuItem.IsChecked = true;
					MediumAlarmMenuItem.IsChecked = false;
					HighAlarmMenuItem.IsChecked = false;
					break;

				case 2:
					NoNotificationMenuItem.IsChecked = false;
					LowAlarmMenuItem.IsChecked = false;
					MediumAlarmMenuItem.IsChecked = true;
					HighAlarmMenuItem.IsChecked = false;
					break;

				case 3:
					NoNotificationMenuItem.IsChecked = false;
					LowAlarmMenuItem.IsChecked = false;
					MediumAlarmMenuItem.IsChecked = false;
					HighAlarmMenuItem.IsChecked = true;
					break;

				default:
					NoNotificationMenuItem.IsChecked = false;
					LowAlarmMenuItem.IsChecked = false;
					MediumAlarmMenuItem.IsChecked = true;
					HighAlarmMenuItem.IsChecked = false;
					break;
			}
		}
	}
}